{%- block doctype -%}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
{%- endblock %}

{%- set url_root = pathto('', 1) %}
{# XXX necessary? #}
{%- if url_root == '#' %}{% set url_root = '' %}{% endif %}

{%- if not embedded and pagename != master_doc and docstitle %}
  {%- set titlesuffix = " &mdash; "|safe + docstitle|e %}
{%- else %}
  {%- set titlesuffix = "" %}
{%- endif %}

{%- macro toc() %}
    <a href="{{ url_root }}">Table of contents</a>
{%- endmacro %}

{%- macro relbar() %}
<div class="relbar">
    <div class="left">
        <a href="{{ site_url }}">Pybtex</a> /
        {%- if pagename != master_doc %}
            <a href="{{ url_root }}">{{ shorttitle|e }}</a> /
            {%- for parent in parents %}
            <a href="{{ parent.link|e }}" {% if loop.last %}{{ accesskey("U") }}{% endif %}>{{ parent.title }}</a> /
            {%- endfor %}
            {{ title }}
        {%- else %}
            {{ shorttitle|e }}
        {%- endif %}
    </div>
    &nbsp;
    <div class="right">
        <form class="search" action="{{ pathto('search') }}" method="get">
        <input type="text" name="q" placeholder="Enter search text..."/>
        <input type="submit" value="{{ _('Search') }}" />
        <input type="hidden" name="check_keywords" value="yes" />
        <input type="hidden" name="area" value="default" />
        </form>
    </div>
</div>
{%- endmacro %}

{%- macro prev_next() %}
<div class="prev-next">
    {%- if prev and prev != master_doc %}
    <div class="left">
        <a href="{{ prev.link }}">← {{ prev.title }}</a>
    </div>
    {%- endif %}

    <div class="middle">
    {%- if pagename != master_doc and next %}
        {{- toc() }}
    {%- else %}
    &nbsp;
    {%- endif %}
    </div>

    <div class="right">
    {%- if next %}
        <a href="{{ next.link }}">{{ next.title }} →</a>
    {%- elif prev %}
        {{- toc() }}
    {%- endif %}
    </div>
</div>
{%- endmacro %}

{%- macro script() %}
    <script type="text/javascript">
      var DOCUMENTATION_OPTIONS = {
        URL_ROOT:    '{{ url_root }}',
        VERSION:     '{{ release|e }}',
        COLLAPSE_INDEX: false,
        FILE_SUFFIX: '{{ '' if no_search_suffix else file_suffix }}',
        HAS_SOURCE:  {{ has_source|lower }}
      };
    </script>
    {%- for scriptfile in script_files %}
    <script type="text/javascript" src="{{ pathto(scriptfile, 1) }}"></script>
    {%- endfor %}
{%- endmacro %}

{%- macro css() %}
    <link rel="stylesheet" href="{{ pathto('_static/' + style, 1) }}" type="text/css" />
    <link rel="stylesheet" href="{{ pathto('_static/pygments.css', 1) }}" type="text/css" />
    {%- for cssfile in css_files %}
    <link rel="stylesheet" href="{{ pathto(cssfile, 1) }}" type="text/css" />
    {%- endfor %}
{%- endmacro %}

<html lang="en" xmlns="http://www.w3.org/1999/xhtml">
  <head>
    <meta http-equiv="Content-Type" content="text/html; charset={{ encoding }}" />
    {{ metatags }}
    {%- block htmltitle %}
    <title>{{ title|striptags|e }}{{ titlesuffix }} {% if not site %}({{ release }}){% endif %}</title>
    {%- endblock %}
    {{ css() }}
    {%- if not site and not embedded %}
    {{ script() }}
    {%- if use_opensearch %}
    <link rel="search" type="application/opensearchdescription+xml"
          title="{% trans docstitle=docstitle|e %}Search within {{ docstitle }}{% endtrans %}"
          href="{{ pathto('_static/opensearch.xml', 1) }}"/>
    {%- endif %}
    {%- if favicon %}
    <link rel="shortcut icon" href="{{ pathto('_static/' + favicon, 1) }}"/>
    {%- endif %}
    {%- endif %}
{%- block linktags %}
    {%- if hasdoc('about') %}
    <link rel="author" title="{{ _('About these documents') }}" href="{{ pathto('about') }}" />
    {%- endif %}
    {%- if hasdoc('genindex') %}
    <link rel="index" title="{{ _('Index') }}" href="{{ pathto('genindex') }}" />
    {%- endif %}
    {%- if hasdoc('search') %}
    <link rel="search" title="{{ _('Search') }}" href="{{ pathto('search') }}" />
    {%- endif %}
    {%- if hasdoc('copyright') %}
    <link rel="copyright" title="{{ _('Copyright') }}" href="{{ pathto('copyright') }}" />
    {%- endif %}
    <link rel="top" title="{{ docstitle|e }}" href="{{ pathto(master_doc) }}" />
    {%- if parents %}
    <link rel="up" title="{{ parents[-1].title|striptags|e }}" href="{{ parents[-1].link|e }}" />
    {%- endif %}
    {%- if next %}
    <link rel="next" title="{{ next.title|striptags|e }}" href="{{ next.link|e }}" />
    {%- endif %}
    {%- if prev %}
    <link rel="prev" title="{{ prev.title|striptags|e }}" href="{{ prev.link|e }}" />
    {%- endif %}
{%- endblock %}
{%- block extrahead %} {% endblock %}
  </head>
  <body role="document">

{%- block content %}

{%- block relbar1 %}
{%- if not site %}{{ relbar() }}{% endif %}
{%- endblock %}

  <div class="content">
    <div class="body" role="main">
    {% block body %} {% endblock %}
    </div>

    {%- block relbar2 %}
    {%- if not site %}{{ prev_next() }}{% endif %}
    {%- endblock %}

    {%- block footer %}
    {%- if not site %}
    <div class="footer" role="contentinfo">
    {%- if show_copyright %}
    {%- if hasdoc('copyright') %}
        {% trans path=pathto('copyright'), copyright=copyright|e %}&copy; <a href="{{ path }}">Copyright</a> {{ copyright }}.{% endtrans %}
    {%- else %}
        {% trans copyright=copyright|e %}&copy; Copyright {{ copyright }}.{% endtrans %}
    {%- endif %}
    {%- endif %}
    {%- if last_updated %}
    {% trans last_updated=last_updated|e %}Last updated on {{ last_updated }}.{% endtrans %}
    {%- endif %}
    {%- if show_sphinx %}
    {% trans sphinx_version=sphinx_version|e %}Created using <a href="http://sphinx-doc.org/">Sphinx</a> {{ sphinx_version }}.{% endtrans %}
    {%- endif %}
    </div>
    {%- endif %}
    {%- endblock %}

  </div>
{%- endblock %}

  </body>
</html>
